/*
 *  lufs.c
 *  
 *  Author  : Ahmed Obied (ahmed.obied@gmail.com)
 *  Version : 0.1
 *
 *  Features: 
 *	1) Create a visual listing of a directory and its subdirectories. 
 *      2) Get information about a directory
 *	3) Search a directory 
 *	4) Delete a file from directory
 *      5) Protect against malicious access to a directory
 *      6) Copy a directory
 *      7) Create a directory and set its modes interactively
 *	
 */
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <stdlib.h>
#include <fcntl.h>
#include <limits.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#define SPACE_SIZE	3
#define FILE_TYPES	7
#define SEC_FILE	".dir_protect"
#define DOT		"."
#define DOT_DOT		".."
#define STR_TERM	'\0'

#define std_error(x) \
{ \
	fprintf(stderr,x); \
	exit(EXIT_FAILURE); \
}

#define pop_error(x) \
{ \
	perror(x); \
	exit(EXIT_FAILURE); \
}

#ifndef _OPTION_TYPES_ENUM_
#define _OPTION_TYPES_ENUM_
enum
{
	_TRAVERSE   = 0  ,
	_SEARCH     = 2  ,
	_GET_STAT   = 4  ,
	_RM         = 6  , 
	_W_PROTECT  = 8  ,
	_R_PROTECT  = 10 ,
	_COPY       = 12 ,
	_MAKE       = 14
};
#endif

#ifndef _FILE_TYPES_ENUM_
#define _FILE_TYPES_ENUM_
enum 
{
	_REG  ,
	_DIR  ,
	_CHR  ,
	_BLK  ,
	_FIFO ,
	_LNK  ,
	_SOCK
};
#endif

char *option_names[] = 
{
	"-t"  , "3" ,
	"-f"  , "4" ,
	"-s"  , "3" ,
	"-r"  , "4" ,
	"-pw" , "3" ,
	"-pr" , "3" ,
	"-c"  , "4" ,
	"-m"  , "3"
};

char *file_names[FILE_TYPES] = 
{
	"regular file"     ,
	"directory"        ,
	"character device" ,
	"block device"     ,
	"fifo"             ,
	"symbolic link"    ,
	"socket"
};

struct file_protection
{
	char file_path[PATH_MAX];
	time_t atime;
	time_t mtime;
	time_t ctime;
};

static void copy_dir(char *, char *, int);
static void remove_dir(char *, char *);
static void get_dir_stat(char *, int *, int);
static void protect_dir(char *, int);
static char get_mod_input(char *msg);
static void make_dir(char *);
static void search_dir(char *, char *, int, int *, char *);
static void traverse_dir(char *dir_name, int);
static void display_usage();
static void protect_traverse(char *, int, int);

/*
 *  protect_traverse
 *
 *  Description:
 *	This function traverses a directory and checks for malicious
 *	access to files
 *
 *  Paramters:
 *	char *dir_name, int choice, int fd
 *
 *  Returns: 
 *	void
 *
 */
static void protect_traverse(char *dir_name, int choice, int fd)
{
	int err;
	DIR *dir_stream;
	struct dirent *dir_entry;
	struct stat f_stat;
		
	dir_stream = opendir(dir_name);
	if (dir_stream == NULL)
		pop_error("protect traverse: opendir error");
	while(1) {
		char *path;
	
		dir_entry = readdir(dir_stream);
		if (dir_entry == NULL)
			break;
		if ((strcmp(dir_entry->d_name,DOT) == 0)
		||
		   (strcmp(dir_entry->d_name,DOT_DOT) == 0))
			continue;
		path = (char *)malloc(PATH_MAX);
		memset(path,STR_TERM,PATH_MAX);
		strcat(path,dir_name);
		strcat(path,"/");
		strcat(path,dir_entry->d_name);
		err = lstat(path,&f_stat);
		if (err < 0)
			pop_error("protect_traverse: lstat error");
		if (choice == _W_PROTECT) {
			struct file_protection f_protect;
			
			memset(&f_protect.file_path,STR_TERM,PATH_MAX);
			memcpy(&f_protect.file_path,path,PATH_MAX);
			f_protect.atime = f_stat.st_atime;
			f_protect.mtime = f_stat.st_mtime;
			f_protect.ctime = f_stat.st_ctime;
			err = write(fd,(char *)&f_protect,sizeof(f_protect));
			if (err < 0)
				pop_error("protect_traverse: write error");
		}
		else
		if (choice == _R_PROTECT) {
			struct file_protection f_protect, t_protect;
			
			memset(&f_protect.file_path,STR_TERM,PATH_MAX);
			memcpy(&f_protect.file_path,path,PATH_MAX);
			f_protect.atime = f_stat.st_atime;
			f_protect.mtime = f_stat.st_mtime;
			f_protect.ctime = f_stat.st_ctime;
			err = read(fd,(char *)&t_protect,sizeof(t_protect));
			if (err < 0)
				pop_error("protect_traverse: read error");
			printf("> Analyzing '%s':\n",path);
			printf("\t- Path: ");
			if (strcmp(f_protect.file_path,t_protect.file_path) == 0)
				printf("OK\n");
			else
				printf("The path was altered\n");
			printf("\t- Access: ");
			if (f_protect.atime == t_protect.atime)
				printf("OK\n");
			else
				printf("Last access time was altered\n");
			printf("\t- Modification: ");
			if (f_protect.mtime == t_protect.mtime)
				printf("OK\n");
			else
				printf("Last modification time was altered\n");
			printf("\t- Change: ");

			if (f_protect.ctime == t_protect.ctime)
				printf("OK\n");
			else
				printf("Last change time was altered\n"); 
			printf("\n");
		}
		if (S_ISDIR(f_stat.st_mode) != 0)
			protect_traverse(path,choice,fd);			
		free(path);
	}	
}

/*
 *  protect_dir
 *
 *  Description:
 *      This function takes a snap shot of a directory to avoid malicious
 *      access to files
 *
 *  Paramters:
 *      char *dir_name, int choice
 *
 *  Returns:
 *      void
 *
 */
static void protect_dir(char *dir_name, int choice)
{
	int fd, err;
	struct stat f_stat;
	
	fd = open(SEC_FILE,O_RDWR|O_CREAT);
	if (fd < 0)
		pop_error("protect_dir: open error");
	err = fchmod(fd,S_IRUSR|S_IWUSR);
	if (err < 0)
		pop_error("protect_dir: fchmod error");
	err = lstat(SEC_FILE,&f_stat);
	if (err < 0)
		pop_error("protect_dir: lstat error");
	if (f_stat.st_size == 0 && choice == _R_PROTECT) {
 		std_error("You need to run the -pw option first.\n");
		return;
	}
	if (f_stat.st_size != 0 && choice == _W_PROTECT) {
		char *cmd;
		
		cmd = (char *)malloc(NAME_MAX);
		memset(cmd,STR_TERM,NAME_MAX);
		strcat(cmd,"cat /dev/null > ");
		strcat(cmd,SEC_FILE);
		err = system(cmd);
		if (err < 0)
			pop_error("protect_dir: system error");
		free(cmd);	
	}
	if (S_ISDIR(f_stat.st_mode) != 0) {
		printf("'%s' is not a directory.\n",dir_name);
		return;
	}
	if (choice == _W_PROTECT) {
		printf("\n> Protecting integrity of '%s' ...\n\n",dir_name);
		protect_traverse(dir_name,_W_PROTECT,fd);		
		printf("Protection data are stored in '%s'.\n\n",SEC_FILE);
	}
	else
	if (choice == _R_PROTECT) {
		printf("\n> Checking integrity of '%s' ...\n\n",dir_name);
		protect_traverse(dir_name,_R_PROTECT,fd);
	}
}

/*
 *  copy_dir
 *
 *  Description:
 *      This function copies a directory and its contents to another
 *      directory
 *
 *  Paramters:
 *      char *o_dir_name, char *n_dir_name, int recursion
 *
 *  Returns:
 *      void
 *
 */
static void copy_dir(char *o_dir_name, char *n_dir_name, int recursion)
{
	int err;
	DIR *dir_stream;
	struct dirent *dir_entry;
	struct stat o_dir_stat;
	
	if (!recursion) {
		printf("\n> Copying '%s' to '%s' ...\n\n",o_dir_name,n_dir_name);
		err = lstat(o_dir_name,&o_dir_stat);
		if (err < 0)
			pop_error("copy_dir: lstat error");
		if (S_ISDIR(o_dir_stat.st_mode) == 0) {
			printf("'%s' is not a directory.\n\n",o_dir_name);
			return;
		} 
		else {
			err = mkdir(n_dir_name,o_dir_stat.st_mode);
			if (err < 0)
				pop_error("copy_dir: mkdir error");
		}
	}
	dir_stream = opendir(o_dir_name);
	if (dir_stream == NULL)
		pop_error("copy_dir: opendir error");
	while (1) {
		char *o_path, *n_path;
	
		dir_entry = readdir(dir_stream);
		if (dir_entry == NULL)
			break;
		if ((strcmp(dir_entry->d_name,DOT) == 0)
	 	||
		   (strcmp(dir_entry->d_name,DOT_DOT) == 0))	
			continue;
		o_path = (char *)malloc(PATH_MAX);	
		n_path = (char *)malloc(PATH_MAX);
		memset(o_path,STR_TERM,PATH_MAX);
		memset(n_path,STR_TERM,PATH_MAX);
		strcat(o_path,o_dir_name);
		strcat(o_path,"/");
		strcat(o_path,dir_entry->d_name);
		strcat(n_path,n_dir_name);
		strcat(n_path,"/");
		strcat(n_path,dir_entry->d_name);
		err = lstat(o_path,&o_dir_stat);
		if (err < 0)
			pop_error("copy_dir: lstat error");
		if (S_ISDIR(o_dir_stat.st_mode) != 0) {
			err = mkdir(n_path,o_dir_stat.st_mode);
			if (err < 0)
				pop_error("copy_dir: mkdir error");
			copy_dir(o_path,n_path,1);
		}
		else {
			int o_fd, n_fd;
			char *data;

			o_fd = open(o_path,O_RDONLY);
			if (o_fd < 0)
				pop_error("copy_dir: open error");
			n_fd = open(n_path,O_RDWR|O_CREAT);
			if (n_fd < 0)
				pop_error("copy_dir: open error");
			err = fstat(o_fd,&o_dir_stat);
			if (err < 0)
				pop_error("copy_dir: fstat error");
			data = (char *)malloc(o_dir_stat.st_size);
			memset(data,STR_TERM,o_dir_stat.st_size);
			err = read(o_fd,data,o_dir_stat.st_size);
			if (err < 0)
				pop_error("copy_dir: read error");
			err = write(n_fd,data,o_dir_stat.st_size);
			if (err < 0)
				pop_error("copy_dir: write error");
			err = fchmod(n_fd,o_dir_stat.st_mode);
			if (err < 0)
				pop_error("copy_dir: fchmod error");
			free(data);
		}
		free(o_path);
		free(n_path);
	}
	if (!recursion)
		printf("'%s' was copied to '%s' successfully.\n\n",
			o_dir_name,n_dir_name);
}

/*
 *  remove_dir
 *
 *  Description:
 *      This function removes a file from a directory      
 *
 *  Paramters:
 *      char *dir_name, char *file_name
 *
 *  Returns:
 *      void
 *
 */
static void remove_dir(char *dir_name, char *file_name)
{
	char path[PATH_MAX];
	int cfound;
	
	cfound = 0;
	printf("\n> Removing '%s' from '%s' ...\n\n",file_name,dir_name);
	while (1) {
		int found, err;
		found = 0;
		memset(&path,STR_TERM,PATH_MAX);
		search_dir(dir_name,file_name,0,&found,path);
		if (found == 0) {
			if (!cfound)
				printf("'%s' was not found.\n\n",file_name);
			break;
		}
		cfound = 1;
		err = remove(path);
		if (err < 0)
			pop_error("remove_dir: remove error");
		printf("'%s' was successfully removed.\n\n",path);
	} 
}

/*
 *  get_dir_stream
 *
 *  Description:
 *      This function get information about a directory 
 *
 *  Paramters:
 *      char *dir_name, int *file_stat, int recursion
 *
 *  Returns:
 *      void
 *
 */
static void get_dir_stat(char *dir_name, int *file_stat, int recursion)
{
	DIR *dir_stream;
	struct dirent *dir_entry;
	struct stat f_stat;
	int err;

	if (!recursion) 
		printf("\n> Retrieving information about '%s' ...\n",dir_name);
	dir_stream = opendir(dir_name);
	if (dir_stream == NULL)
		pop_error("get_dir_stat: opendir error");
	while (1) {
		char *path;
		
		dir_entry = readdir(dir_stream);
		if (dir_entry == NULL)
			break;
		if ((strcmp(dir_entry->d_name,DOT) == 0)
		||
		   (strcmp(dir_entry->d_name,DOT_DOT) == 0))
			continue;
		path = (char *)malloc(PATH_MAX);
		memset(path,STR_TERM,PATH_MAX);
		strcat(path,dir_name);
		strcat(path,"/");
		strcat(path,dir_entry->d_name);
		err = lstat(path,&f_stat);
		if (err < 0)
			pop_error("get_dir_stat: lstat error");
		if (S_ISREG(f_stat.st_mode) != 0) 
			++file_stat[_REG];
		else
		if (S_ISDIR(f_stat.st_mode) != 0) {
			++file_stat[_DIR];
			get_dir_stat(path,file_stat,1);
		}
		else
		if (S_ISCHR(f_stat.st_mode) != 0)
			++file_stat[_CHR];
		else
		if (S_ISBLK(f_stat.st_mode) != 0)
			++file_stat[_BLK];
		else
		if (S_ISFIFO(f_stat.st_mode) != 0)
			++file_stat[_FIFO];
		else
		if (S_ISLNK(f_stat.st_mode) != 0)
			++file_stat[_LNK];	
		else
		if (S_ISSOCK(f_stat.st_mode) != 0)
			++file_stat[_SOCK];
		free(path);
	}
	if (!recursion) {
		int total;
	
		memset((char *)&f_stat,STR_TERM,sizeof(struct stat));
		err = stat(dir_name,&f_stat);
		if (err < 0)
			pop_error("get_dir_stat: stat error"); 
		total = file_stat[_REG] + file_stat[_DIR] + file_stat[_CHR]  +
			file_stat[_LNK] + file_stat[_BLK] + file_stat[_FIFO] +
			file_stat[_SOCK]; 
		printf("\n");
		printf("Time of last access               : %s",
			ctime(&f_stat.st_atime));
		printf("Time of last modification         : %s",
			ctime(&f_stat.st_mtime));
		printf("Time of last change               : %s",
			ctime(&f_stat.st_ctime));
		printf("User ID of owner                  : %d\n",
			(int)f_stat.st_uid);
		printf("Group ID of owner                 : %d\n",
			(int)f_stat.st_gid);
		printf("Total size in bytes               : %d\n",
			(int)f_stat.st_size);
		printf("Total number of files             : %d\n",total);
		printf("Total number of regular files     : %d\n",file_stat[_REG]);
		printf("Total number of directories       : %d\n",file_stat[_DIR]);
		printf("Total number of symbolic links    : %d\n",file_stat[_LNK]);
		printf("Total number of character devices : %d\n",file_stat[_CHR]);
		printf("Total number of block devices     : %d\n",file_stat[_BLK]);
		printf("Total number of fifos             : %d\n",file_stat[_FIFO]);
		printf("Total number of sockets           : %d\n",file_stat[_SOCK]);
		printf("\n");
	}
}

/*
 *  get_mod_input
 *
 *  Description:
 *      This function gets the mode info from stdin
 *
 *  Paramters:
 *      char *msg
 *
 *  Returns:
 *      char
 *
 */
static char get_mod_input(char *msg) 
{
	char choice, skip;
	
	printf("%s",msg);
	choice = (char)getchar();
	if (choice != '\n') {
		do 
			skip = (char)getchar();
		while (skip != '\n');
	}
	return choice;
}

/*
 *  make_dir
 *
 *  Description:
 *      This function creates a directory and sets its modes interactively
 *
 *  Paramters:
 *      char *dir_name
 *
 *  Returns:
 *      void
 *
 */
static void make_dir(char *dir_name)
{
	int err;
	char choice;
	mode_t mode;
	
	printf("\n> Attempting to create directory '%s' ...\n",dir_name);
	
	err = mkdir(dir_name,0);
	if (err < 0)
		pop_error("make_dir: mkdir error");
	printf("\n '%s' was created successfully.\n",dir_name);
	printf("\n> Attempting to set mode for '%s' ...\n",dir_name);
	mode  = 0;
	choice = ' ';	
	printf("\n");
	choice = get_mod_input("Set user ID on execution? (y/n)(default=n): ");
	if (choice == 'y') 
		mode |= S_ISUID;
	choice = get_mod_input("Set group ID on execution? (y/n)(default=n): ");
	if (choice == 'y')
		mode |= S_ISGID;
	choice = get_mod_input("Set sticky bit? (y/n)(default=n): ");
	if (choice == 'y')
		mode |= S_ISVTX;
	choice = get_mod_input("Read by owner? (y/n)(default=y): ");
	if (choice != 'n')
		mode |= S_IRUSR;
	choice = get_mod_input("Write by owner? (y/n)(default=y): ");
	if (choice != 'n')
		mode |= S_IWUSR;
	choice = get_mod_input("Execute/Search by owner? (y/n)(default=n): ");
	if (choice == 'y')
		mode |= S_IXUSR;
	choice = get_mod_input("Read by group? (y/n)(default=n): ");
	if (choice == 'y')
		mode |= S_IRGRP;
	choice = get_mod_input("Write by group? (y/n)(default=n): ");
	if (choice == 'y')
		mode |= S_IWGRP;
	choice = get_mod_input("Execute/Search by group? (y/n)(default=n): ");
	if (choice == 'y')
		mode |= S_IXGRP;
	choice = get_mod_input("Read by others? (y/n)(default=n): ");
	if (choice == 'y')
		mode |= S_IROTH;
	choice = get_mod_input("Write by others? (y/n)(default=n): ");
	if (choice == 'y')
		mode |= S_IWOTH;
	choice = get_mod_input("Execute/Search by others? (y/n)(default=n): ");
	if (choice == 'y')
		mode |= S_IXOTH;
	err = chmod(dir_name,mode);
	if (err < 0)
		pop_error("make_dir: chmod error"); 
	printf("\nModes for '%s' were set successfully.\n\n",dir_name);
}

/*
 *  search_dir
 *
 *  Description:
 *      This function searches for a file name in a directory 
 *
 *  Paramters:
 *      char *dir_name, char *file_name, int recursion, int *found, char *cpath
 *
 *  Returns:
 *      void
 *
 */
static void search_dir(char *dir_name, char *file_name, int recursion, int *found,
		       char *cpath)
{
	DIR *dir_stream;
	struct dirent *dir_entry;
	int err;

	if (!recursion && cpath == NULL)	
		printf("\n> Searching for '%s' in '%s' ...\n\n",file_name,dir_name);
	dir_stream = opendir(dir_name);
	if (dir_stream == NULL)
		pop_error("search_dir: opendir error");
	while (1) {
		char *path;
		struct stat f_stat;
	
		dir_entry = readdir(dir_stream);
		if (dir_entry == NULL)
			break;
		if ((strcmp(dir_entry->d_name,DOT) == 0)
		||
		   (strcmp(dir_entry->d_name,DOT_DOT) == 0))
			continue;
		path = (char *)malloc(PATH_MAX);
		memset(path,STR_TERM,PATH_MAX);
		strcat(path,dir_name);
		strcat(path,"/");
		strcat(path,dir_entry->d_name);
		if (strcmp(file_name,dir_entry->d_name) == 0) {
			if (cpath != NULL) 
				memcpy(cpath,path,PATH_MAX);
			else
				printf("Path: '%s'\n",path);
			*found = 1;
			free(path);
		} 
		else {
			err = lstat(path,&f_stat);
			if (err < 0) 
				pop_error("search_dir: lstat error");
			if (S_ISDIR(f_stat.st_mode) != 0) 
				search_dir(path,file_name,1,found,cpath);	
		}
	}
	if (!recursion) {
		if (*found) {
			if (cpath == NULL) 
				printf("\n'%s' was found.\n\n",file_name);
		}
		else
			if (cpath == NULL)
				printf("\n'%s' was not found.\n\n",file_name);
	}
}

/*
 *  traverse_dir
 *
 *  Description:
 *      This function creates a visual listing of a directory and its
 *  	subdirectories
 *
 *  Paramters:
 *      char *dir_name, int space
 *
 *  Returns:
 *      void
 *
 */
static void traverse_dir(char *dir_name, int space)
{	
	DIR *dir_stream;
	struct dirent *dir_entry;
	int err;

	dir_stream = opendir(dir_name);
	if (dir_stream == NULL) 
		pop_error("traverse_dir: opendir error");
	while (1) {
		int i;
		char *path;
		struct stat f_stat;
		
		dir_entry = readdir(dir_stream);
		if (dir_entry == NULL)
			break;
		if ((strcmp(dir_entry->d_name,DOT) == 0) 
		||
		   (strcmp(dir_entry->d_name,DOT_DOT) == 0))
			continue;
		path = (char *)malloc(PATH_MAX);
		memset(path,STR_TERM,PATH_MAX);
		strcat(path,dir_name);
		strcat(path,"/");
		strcat(path,dir_entry->d_name);
		err = lstat(path,&f_stat);
		if (err < 0) 			
			pop_error("traverse_dir: lstat error");
		if (S_ISDIR(f_stat.st_mode) != 0)
			for (i = 0 ; i < space ; i++)
				printf("-");
		else
			for (i = 0 ; i < space ; i++)
				printf(" ");
		printf("| %s ",dir_entry->d_name);
		if (S_ISREG(f_stat.st_mode) != 0)
			printf("(%s)\n",file_names[_REG]);
		else
		if (S_ISDIR(f_stat.st_mode) != 0) {
			space = space + SPACE_SIZE;
			printf("(%s)\n",file_names[_DIR]);
			traverse_dir(path,space);
			space = space - SPACE_SIZE;
		}
		else
		if (S_ISCHR(f_stat.st_mode) != 0)
			printf("(%s)\n",file_names[_CHR]);
		else
		if (S_ISBLK(f_stat.st_mode) != 0)
			printf("(%s)\n",file_names[_BLK]);
		else
		if (S_ISFIFO(f_stat.st_mode) != 0)
			printf("(%s)\n",file_names[_FIFO]);
		else
		if (S_ISLNK(f_stat.st_mode) != 0) {
			char *points_to, *absolute_path;
			
			points_to = (char *)malloc(PATH_MAX);
			absolute_path = (char *)malloc(PATH_MAX);
			memset(points_to,STR_TERM,PATH_MAX);
			memset(absolute_path,STR_TERM,PATH_MAX);
			err = readlink(path,points_to,PATH_MAX);
			if (err < 0) 
				pop_error("traverse_dir: readlink error");
			if (realpath(path,absolute_path) == NULL)
				pop_error("traverse_dir: realpath error");
			printf("(%s - points to '%s', absolute path : '%s')\n",
				file_names[_LNK],points_to,absolute_path);
			free(points_to);
			free(absolute_path);
		}
		else
		if (S_ISSOCK(f_stat.st_mode) != 0)
			printf("(%s)\n",file_names[_SOCK]);
		free(path);
	}
	err = closedir(dir_stream);
	if (err < 0)
		pop_error("traverse_dir: closedir error");
}

/*
 *  display_usage
 *
 *  Description:
 *      This function displays the usage menu to the user
 *
 *  Paramters:
 *      None
 *
 *  Returns:
 *      void
 *
 */
static void display_usage()
{
	printf("\nUsage: './fs [option]'\n");
	printf("\n[option]:\n\n");
	printf("-t [directory name]\n");
	printf("(To traverse 'directory name')\n\n");
	printf("-f [directory name] [file name]\n");
	printf("(To search for 'file name' in 'directory name')\n\n");
	printf("-s [directory name]\n");
	printf("(To get statistics about 'directory name')\n\n"); 
	printf("-r [directory name] [name]\n");
	printf("(To remove 'name' from 'directory name')\n\n");
	printf("-pw [directory name]\n");
	printf("(To protect the integrity of 'directory name')\n\n");
	printf("-pr [directory name]\n");
	printf("(To check integrity protection of 'directory name')\n\n");
	printf("-c [old directory] [new directory]\n");
	printf("(To copy the contents of 'old directory' to 'new directory')\n\n");
	printf("-m [directory name]\n");
	printf("(To create 'directory name')\n\n");
}

/*
 *  main
 *
 *  Description:
 *      This is the entry function of the program
 *
 *  Paramters:
 *      int argc, char **argv
 *
 *  Returns:
 *      int
 *
 */
int main(int argc, char **argv)
{
	if (argc < 2) {
		display_usage();
		return 0;
	}
	if (strcmp(argv[1],option_names[_TRAVERSE]) == 0) {
		if (argc == atoi(option_names[_TRAVERSE + 1])) { 
			traverse_dir(argv[2],0);
			return 0;
		}
		else
			std_error("Usage: './fs -t [directory name]'\n");
	}
	if (strcmp(argv[1],option_names[_SEARCH]) == 0) {
		if (argc == atoi(option_names[_SEARCH + 1])) {
			int found;
	
			found = 0;
			search_dir(argv[2],argv[3],0,&found,NULL);
			return 0;
		}
		else
			std_error("Usage: './fs -f [directory name] [file name]'\n");
	}
	if (strcmp(argv[1],option_names[_GET_STAT]) == 0) {
		if (argc == atoi(option_names[_GET_STAT + 1])) {
			int *file_stat;
			
			file_stat = (int *)malloc(FILE_TYPES * sizeof(int));
			memset(file_stat,0,FILE_TYPES * sizeof(int));
			get_dir_stat(argv[2],file_stat,0);
			free(file_stat);
			return 0;
		}
		else
			std_error("Usage: './fs -s [directory name]'\n");
	}
	if (strcmp(argv[1],option_names[_RM]) == 0) {
		if (argc == atoi(option_names[_RM + 1])) {
			remove_dir(argv[2],argv[3]);
			return 0;
		}
		else
			std_error("Usage: './fs -r [directory name] [name]'\n");
	}
	if (strcmp(argv[1],option_names[_W_PROTECT]) == 0) {
		if (argc == atoi(option_names[_W_PROTECT + 1])) {
			protect_dir(argv[2],_W_PROTECT);
			return 0;
		}
		else
			std_error("Usage: './fs -pw [directory name]'\n");
	}
	if (strcmp(argv[1],option_names[_R_PROTECT]) == 0) {
		if (argc == atoi(option_names[_R_PROTECT + 1])) {
			protect_dir(argv[2],_R_PROTECT);
			return 0;
		}
		else
			std_error("Usage: './fs -pr [directory name]'\n");
	}
	else
	if (strcmp(argv[1],option_names[_COPY]) == 0) {
		if (argc == atoi(option_names[_COPY + 1])) {
			copy_dir(argv[2],argv[3],0);
			return 0;
		}
		else
			std_error("Usage: './fs -c [old directory][new directory]'\n");
	}
	if (strcmp(argv[1],option_names[_MAKE]) == 0) {
		if (argc == atoi(option_names[_MAKE + 1])) {
			make_dir(argv[2]);
			return 0;
		}
		else
			std_error("Usage: './fs -m [directory name]'\n");
	}
	display_usage();
	return EXIT_SUCCESS;
}



